#include <SPI.h>
#include <LoRa.h>
#include <Wire.h>  
#include "SSD1306.h" 

//pino Botão
#define PIN_BTN 23 
//pino LED
#define PIN_LED 22

//armazenam estado do botão
int estadoBtn, ultimoEstadoBtn;

// Pin definetion of WIFI LoRa 32
// HelTec AutoMation 2017 support@heltec.cn 
#define SCK     5    // GPIO5  -- SX127x's SCK
#define MISO    19   // GPIO19 -- SX127x's MISO
#define MOSI    27   // GPIO27 -- SX127x's MOSI
#define SS      18   // GPIO18 -- SX127x's CS
#define RST     14   // GPIO14 -- SX127x's RESET
#define DI00    26   // GPIO26 -- SX127x's IRQ(Interrupt Request)

#define PABOOST true

//array de frequencias válidas que utilizaremos para trocar
float frequencia[6] = {915E6, 915.125E6, 868E6, 868.125E6, 433E6, 433.125E6};
//controla o indice da frequencia atual no array 
int indice = 0;

unsigned int counter = 0;

SSD1306 display(0x3c, 4, 15);

void setup() {
  pinMode(16,OUTPUT);
  pinMode(2,OUTPUT);

  Serial.begin(115200);

  pinMode(PIN_BTN, INPUT);
  pinMode(PIN_LED, OUTPUT);
  
  digitalWrite(16, LOW);    // set GPIO16 low to reset OLED
  delay(50); 
  digitalWrite(16, HIGH); // while OLED is running, must set GPIO16 in high

  display.init();
  display.flipScreenVertically();  
  display.setFont(ArialMT_Plain_10);
  delay(1500);
  display.clear();
  
  SPI.begin(SCK,MISO,MOSI,SS);
  LoRa.setPins(SS,RST,DI00);
  
  if (!LoRa.begin(frequencia[indice],PABOOST))
  {
    display.drawString(0, 0, "Starting LoRa failed!");
    display.display();
    while (1);
  }
  LoRa.enableCrc();
  display.drawString(0, 0, "LoRa Initial success!");
  display.display();
  delay(1000);

  //define estado do botao como desligado
  estadoBtn = ultimoEstadoBtn = LOW;
  digitalWrite(PIN_LED, LOW);

}

void loop() 
{
  //recupera o estado do botao
  estadoBtn = digitalRead(PIN_BTN);
  //verifica se o estado do botão mudou e se mudou para LIGADO
  if( (estadoBtn != ultimoEstadoBtn) && (estadoBtn == HIGH) )
  {
    //liga o LED indicador de mudança de frequencia
    digitalWrite(PIN_LED, HIGH);  
    //verificação para não estourar o indice do array   
     if( (indice+1) > 5){ 
        indice = 0; 
     }
     else indice++; 
     
     enviarPacote();    
     atualizarDisplay();
     mudarFrequencia();     
  }
  //caso nao tenha apertado o botão, ou seja, seu estado é DESLIGADO, então a frequencia permanece a mesma
  else {
    enviarPacote();  
    atualizarDisplay();
  }
  //contador que é enviado (DADO)
  counter++;

  //copia o estado atual do botão na variável de verificação da mudança de estado
  ultimoEstadoBtn = estadoBtn;

  //acende LED interno para indicar que enviou o pacote e da um delay
  digitalWrite(2, HIGH);   // turn the LED on (HIGH is the voltage level)
  delay(700);               // wait for a second
  digitalWrite(2, LOW);    // turn the LED off by making the voltage LOW
  delay(500);               // wait for a second
}

//muda a frequencia do Lora, a nova frequencia será de acordo com a variável "INDICE" que pegará no array a nova frequencia
void mudarFrequencia()
{
  if (!LoRa.begin(frequencia[indice],PABOOST))
  { 
    display.drawString(0, 0, "Starting LoRa failed!");
    display.display();
    while (1);
  }
  LoRa.enableCrc();
  digitalWrite(PIN_LED, LOW); //desliga o LED indicador da mudança de frequência
}

//envia o pacote na rede Lora
void enviarPacote()
{
  // envia u pacote
  LoRa.beginPacket(); 
  LoRa.print(indice); //indice do array que contem as frequencias
  LoRa.print("#"); //delimitador
  LoRa.print(counter); //dado
  LoRa.endPacket();
}

//atualiza as informações no display
void atualizarDisplay()
{
  display.clear();
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.setFont(ArialMT_Plain_10);
  
  display.drawString(0, 0, "Enviando Pacote: ");
  display.drawString(90, 0, String(counter));
  display.drawString(0, 20, String(frequencia[indice]));
  display.drawString(90, 20, "I: "+String(indice));  
  display.display();
}

